from opentrons.protocol_api import COLUMN, ALL, Parameters

metadata = {
    'protocolName': 'Flex Demo Protocol',
    'author': 'Nick Diehl <ndiehl@opentrons.com>',
}
requirements = {
    'robotType': 'Flex',
    'apiLevel': '2.18'
}

def add_parameters(parameters: Parameters):

    parameters.add_int(
        variable_name="num_samples",
        display_name="Number of samples",
        description="Number of samples in plate",
        default=48,
        minimum=8,
        maximum=96,
    )

    parameters.add_float(
    variable_name="volume",
    display_name="starting sample volume",
    description="Volume (in uL) for each sample.",
    default=5.0,
    choices=[
            {"display_name": "Low (2.5 µL)", "value": 2.5},
            {"display_name": "Medium (5 µL)", "value": 5.0},
            {"display_name": "High (10 µL)", "value": 10.0},
        ]
    )

    parameters.add_bool(
        variable_name="use_hs",
        display_name="Activate heater-shaker",
        description="Activate heater shaker during protocol",
        default=True
    )

def run(ctx):

    ctx.comment(f'Number of samples selected: {ctx.params.num_samples}')
    ctx.comment(f'Volume of sample: {ctx.params.volume}')
    hs_string = 'yes' if ctx.params.use_hs else 'no'
    ctx.comment(f'Heater shaker activated: {hs_string}')

    # deck
    tc = ctx.load_module('thermocyclerModuleV2')
    hs = ctx.load_module('heaterShakerModuleV1', 'D1')
    hs_adapter = hs.load_adapter('opentrons_96_deep_well_adapter')
    # deepwell = hs_adapter.load_labware(
    #     'nest_96_wellplate_2ml_deep', 'nicks plate')
    deepwell = ctx.load_labware(
        'nest_96_wellplate_2ml_deep', 'C1', 'nicks plate')
    hs.close_labware_latch()
    magblock = ctx.load_module('magneticBlockV1', 'D2')
    deepwell = magblock.load_labware(
        'nest_96_wellplate_2ml_deep', 'mag plate')
    tempdeck = ctx.load_module('temperatureModuleV2', 'D3')
    adapter = tempdeck.load_adapter('opentrons_96_well_aluminum_block')
    tempplate = adapter.load_labware(
        'opentrons_96_wellplate_200ul_pcr_full_skirt' , 'nick temp plate')
    res1 = ctx.load_adapter('opentrons_96_well_aluminum_block', 'C4')
    res1_lw = res1.load_labware('opentrons_96_wellplate_200ul_pcr_full_skirt')
    res12 = ctx.load_labware('nest_12_reservoir_15ml', 'C3')
    res_waste = ctx.load_labware('nest_1_reservoir_195ml', 'A2')
    tiprack_8 = ctx.load_labware('opentrons_flex_96_tiprack_200ul', 'B2')
    tiprack_96 = ctx.load_labware(
        'opentrons_flex_96_tiprack_200ul',
        'B3',
        adapter='opentrons_flex_96_tiprack_adapter'
    )
    tc.load_labware('opentrons_96_wellplate_200ul_pcr_full_skirt', 'tc plate')
    ctx.load_trash_bin('A3')

    liq = ctx.define_liquid(name='liq', description='', display_color='#FFFF00')
    [well.load_liquid(liq, volume=100) for well in deepwell.wells()]

    # instruments
    p1000_96 = ctx.load_instrument('flex_96channel_1000')
    p1000_96.configure_nozzle_layout(
        style=ALL,
        tip_racks=[tiprack_96]
    )
    p1000_96.transfer(100, res12.wells()[0], deepwell.wells()[0], trash=False)

    hs.close_labware_latch()
    hs.set_and_wait_for_shake_speed(500)
    ctx.delay(seconds=10, msg='shaking')
    hs.deactivate_shaker()
    hs.open_labware_latch()

    # ctx.move_labware(deepwell, magblock, use_gripper=True)

    ctx.delay(seconds=10)

    p1000_96.reset_tipracks()
    p1000_96.transfer(
        100, deepwell.wells()[0], res_waste.wells()[0].top(), trash=False)

    ctx.move_labware(deepwell, hs_adapter, use_gripper=True)

    p1000_96.configure_nozzle_layout(
        style=COLUMN,
        start='A12',
        tip_racks=[tiprack_8]
    )

    hs.close_labware_latch()
    p1000_96.distribute(
        15,
        res12.wells()[0],
        [col.top() for col in deepwell.rows()[0][6:]],
        disposal_vol=0,
    )

    p1000_96.configure_nozzle_layout(
        style=ALL,
        tip_racks=[tiprack_96]
    )
    p1000_96.reset_tipracks()

    p1000_96.transfer(
        10,
        deepwell.wells()[0],
        tempplate.wells()[0],
        trash=False
    )

    tc.open_lid()

    # ctx.move_labware(tempplate, tc, use_gripper=True)
